@@ -18,7 +18,7 @@
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
 
 """check for signs of poor design"""
 
@@ -212,7 +212,7 @@ class MisdesignChecker(BaseChecker):
                 "default": 50,
                 "type": "int",
                 "metavar": "<int>",
-                "help": "Maximum number of statements in function / method " "body.",
+                "help": "Maximum number of statements in function / method body.",
             },
         ),
         (
