@@ -17,15 +17,18 @@
 # Copyright (c) 2020 Ganden Schaffner <gschaffner@pm.me>
 # Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 Damien Baty <damien.baty@polyconseil.fr>
+# Copyright (c) 2021 Andreas Finkler <andi.finkler@gmail.com>
+# Copyright (c) 2021 Eli Fine <ejfine@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
 
 """Checker for spelling errors in comments and docstrings.
 """
 import os
 import re
 import tokenize
+from typing import Pattern
 
 from pylint.checkers import BaseTokenChecker
 from pylint.checkers.utils import check_messages
@@ -79,7 +82,7 @@ else:
     instr = " To make it work, install the 'python-enchant' package."
 
 
-class WordsWithDigigtsFilter(Filter):
+class WordsWithDigitsFilter(Filter):
     """Skips words with digits."""
 
     def _skip(self, word):
@@ -99,7 +102,21 @@ class WordsWithUnderscores(Filter):
         return "_" in word
 
 
-class CamelCasedWord(Filter):
+class RegExFilter(Filter):
+    """Parent class for filters using regular expressions.
+
+    This filter skips any words the match the expression
+    assigned to the class attribute ``_pattern``.
+
+    """
+
+    _pattern: Pattern[str]
+
+    def _skip(self, word) -> bool:
+        return bool(self._pattern.match(word))
+
+
+class CamelCasedWord(RegExFilter):
     r"""Filter skipping over camelCasedWords.
     This filter skips any words matching the following regular expression:
 
@@ -109,11 +126,8 @@ class CamelCasedWord(Filter):
     """
     _pattern = re.compile(r"^([a-z]+([\d]|[A-Z])(?:\w+)?)")
 
-    def _skip(self, word):
-        return bool(self._pattern.match(word))
 
-
-class SphinxDirectives(Filter):
+class SphinxDirectives(RegExFilter):
     r"""Filter skipping over Sphinx Directives.
     This filter skips any words matching the following regular expression:
 
@@ -124,11 +138,8 @@ class SphinxDirectives(Filter):
     # The final ` in the pattern is optional because enchant strips it out
     _pattern = re.compile(r"^(:([a-z]+)){1,2}:`([^`]+)(`)?")
 
-    def _skip(self, word):
-        return bool(self._pattern.match(word))
 
-
-class ForwardSlashChunkder(Chunker):
+class ForwardSlashChunker(Chunker):
     """
     This chunker allows splitting words like 'before/after' into 'before' and 'after'
     """
@@ -169,6 +180,23 @@ class ForwardSlashChunkder(Chunker):
         raise StopIteration()
 
 
+CODE_FLANKED_IN_BACKTICK_REGEX = re.compile(r"(\s|^)(`{1,2})([^`]+)(\2)([^`]|$)")
+
+
+def _strip_code_flanked_in_backticks(line: str) -> str:
+    """Alter line so code flanked in backticks is ignored.
+
+    Pyenchant automatically strips backticks when parsing tokens,
+    so this cannot be done at the individual filter level."""
+
+    def replace_code_but_leave_surrounding_characters(match_obj) -> str:
+        return match_obj.group(1) + match_obj.group(5)
+
+    return CODE_FLANKED_IN_BACKTICK_REGEX.sub(
+        replace_code_but_leave_surrounding_characters, line
+    )
+
+
 class SpellingChecker(BaseTokenChecker):
     """Check spelling in comments and docstrings"""
 
@@ -211,7 +239,7 @@ class SpellingChecker(BaseTokenChecker):
                 "default": "",
                 "type": "string",
                 "metavar": "<comma separated words>",
-                "help": "List of comma separated words that " "should not be checked.",
+                "help": "List of comma separated words that should not be checked.",
             },
         ),
         (
@@ -242,7 +270,16 @@ class SpellingChecker(BaseTokenChecker):
                 "default": 4,
                 "type": "int",
                 "metavar": "N",
-                "help": "Limits count of emitted suggestions for " "spelling mistakes.",
+                "help": "Limits count of emitted suggestions for spelling mistakes.",
+            },
+        ),
+        (
+            "spelling-ignore-comment-directives",
+            {
+                "default": "fmt: on,fmt: off,noqa:,noqa,nosec,isort:skip,mypy:",
+                "type": "string",
+                "metavar": "<comma separated words>",
+                "help": "List of comma separated words that should be considered directives if they appear and the beginning of a comment and should not be checked.",
             },
         ),
     )
@@ -264,6 +301,10 @@ class SpellingChecker(BaseTokenChecker):
         # "pylint" appears in comments in pylint pragmas.
         self.ignore_list.extend(["param", "pylint"])
 
+        self.ignore_comment_directive_list = [
+            w.strip() for w in self.config.spelling_ignore_comment_directives.split(",")
+        ]
+
         # Expand tilde to allow e.g. spelling-private-dict-file = ~/.pylintdict
         if self.config.spelling_private_dict_file:
             self.config.spelling_private_dict_file = os.path.expanduser(
@@ -274,7 +315,9 @@ class SpellingChecker(BaseTokenChecker):
             self.spelling_dict = enchant.DictWithPWL(
                 dict_name, self.config.spelling_private_dict_file
             )
-            self.private_dict_file = open(self.config.spelling_private_dict_file, "a")
+            self.private_dict_file = open(  # pylint: disable=consider-using-with
+                self.config.spelling_private_dict_file, "a"
+            )
         else:
             self.spelling_dict = enchant.Dict(dict_name)
 
@@ -283,12 +326,12 @@ class SpellingChecker(BaseTokenChecker):
 
         self.tokenizer = get_tokenizer(
             dict_name,
-            chunkers=[ForwardSlashChunkder],
+            chunkers=[ForwardSlashChunker],
             filters=[
                 EmailFilter,
                 URLFilter,
                 WikiWordFilter,
-                WordsWithDigigtsFilter,
+                WordsWithDigitsFilter,
                 WordsWithUnderscores,
                 CamelCasedWord,
                 SphinxDirectives,
@@ -308,9 +351,19 @@ class SpellingChecker(BaseTokenChecker):
             initial_space = 0
         if line.strip().startswith("#") and "docstring" not in msgid:
             line = line.strip()[1:]
+            # A ``Filter`` cannot determine if the directive is at the beginning of a line,
+            #   nor determine if a colon is present or not (``pyenchant`` strips trailing colons).
+            #   So implementing this here.
+            for iter_directive in self.ignore_comment_directive_list:
+                if line.startswith(" " + iter_directive):
+                    line = line[(len(iter_directive) + 1) :]
+                    break
             starts_with_comment = True
         else:
             starts_with_comment = False
+
+        line = _strip_code_flanked_in_backticks(line)
+
         for word, word_start_at in self.tokenizer(line.strip()):
             word_start_at += initial_space
             lower_cased_word = word.casefold()
