@@ -44,7 +44,7 @@
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/PyCQA/pylint/blob/master/LICENSE
 
 """some functions that may be useful for various checkers
 """
@@ -70,8 +70,6 @@ from typing import (
 import _string
 import astroid
 
-from pylint.constants import PY310_PLUS
-
 BUILTINS_NAME = builtins.__name__
 COMP_NODE_TYPES = (
     astroid.ListComp,
@@ -251,7 +249,7 @@ SUBSCRIPTABLE_CLASSES_PEP585 = frozenset(
         "_collections_abc.Container",
         "_collections_abc.Collection",
         "_collections_abc.Callable",
-        "_collections_abc.Set # typing.AbstractSet",
+        "_collections_abc.Set",
         "_collections_abc.MutableSet",
         "_collections_abc.Mapping",
         "_collections_abc.MutableMapping",
@@ -867,7 +865,7 @@ def unimplemented_abstract_methods(
     """
     if is_abstract_cb is None:
         is_abstract_cb = partial(decorated_with, qnames=ABC_METHODS)
-    visited = {}  # type: Dict[str, astroid.node_classes.NodeNG]
+    visited: Dict[str, astroid.node_classes.NodeNG] = {}
     try:
         mro = reversed(node.mro())
     except NotImplementedError:
@@ -969,7 +967,7 @@ def _except_handlers_ignores_exception(
     handlers: astroid.ExceptHandler, exception
 ) -> bool:
     func = partial(error_of_type, error_type=(exception,))
-    return any(map(func, handlers))
+    return any(func(handler) for handler in handlers)
 
 
 def get_exception_handlers(
@@ -1328,9 +1326,6 @@ def get_node_last_lineno(node: astroid.node_classes.NodeNG) -> int:
 
 def is_postponed_evaluation_enabled(node: astroid.node_classes.NodeNG) -> bool:
     """Check if the postponed evaluation of annotations is enabled"""
-    if PY310_PLUS:
-        return True
-
     module = node.root()
     return "annotations" in module.future_imports
 
@@ -1341,22 +1336,41 @@ def is_class_subscriptable_pep585_with_postponed_evaluation_enabled(
     """Check if class is subscriptable with PEP 585 and
     postponed evaluation enabled.
     """
-    if not is_postponed_evaluation_enabled(node):
-        return False
+    return (
+        is_postponed_evaluation_enabled(node)
+        and value.qname() in SUBSCRIPTABLE_CLASSES_PEP585
+        and is_node_in_type_annotation_context(node)
+    )
 
-    parent_node = node.parent
+
+def is_node_in_type_annotation_context(node: astroid.node_classes.NodeNG) -> bool:
+    """Check if node is in type annotation context.
+
+    Check for 'AnnAssign', function 'Arguments',
+    or part of function return type anntation.
+    """
+    # pylint: disable=too-many-boolean-expressions
+    current_node, parent_node = node, node.parent
     while True:
-        # Check if any parent node matches condition
-        if isinstance(
-            parent_node, (astroid.AnnAssign, astroid.Arguments, astroid.FunctionDef)
+        if (
+            isinstance(parent_node, astroid.AnnAssign)
+            and parent_node.annotation == current_node
+            or isinstance(parent_node, astroid.Arguments)
+            and current_node
+            in (
+                *parent_node.annotations,
+                *parent_node.posonlyargs_annotations,
+                *parent_node.kwonlyargs_annotations,
+                parent_node.varargannotation,
+                parent_node.kwargannotation,
+            )
+            or isinstance(parent_node, astroid.FunctionDef)
+            and parent_node.returns == current_node
         ):
-            break
-        parent_node = parent_node.parent
+            return True
+        current_node, parent_node = parent_node, parent_node.parent
         if isinstance(parent_node, astroid.Module):
             return False
-    if value.qname() in SUBSCRIPTABLE_CLASSES_PEP585:
-        return True
-    return False
 
 
 def is_subclass_of(child: astroid.ClassDef, parent: astroid.ClassDef) -> bool:
@@ -1459,8 +1473,12 @@ def is_attribute_typed_annotation(
     return False
 
 
-def is_class_var(node: astroid.AssignName) -> bool:
-    """Test if node has `ClassVar` annotation."""
+def is_assign_name_annotated_with(node: astroid.AssignName, typing_name: str) -> bool:
+    """Test if AssignName node has `typing_name` annotation.
+
+    Especially useful to check for `typing._SpecialForm` instances
+    like: `Union`, `Optional`, `Literal`, `ClassVar`, `Final`.
+    """
     if not isinstance(node.parent, astroid.AnnAssign):
         return False
     annotation = node.parent.annotation
@@ -1468,9 +1486,9 @@ def is_class_var(node: astroid.AssignName) -> bool:
         annotation = annotation.value
     if (
         isinstance(annotation, astroid.Name)
-        and annotation.name == "ClassVar"
+        and annotation.name == typing_name
         or isinstance(annotation, astroid.Attribute)
-        and annotation.attrname == "ClassVar"
+        and annotation.attrname == typing_name
     ):
         return True
     return False
